/*
 * $Id: rb_software_x11.c 40 2004-07-25 13:14:34Z tilman $
 *
 * Copyright (C) 2004 Tilman Sauerbeck (tilman at code-monkey de)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <ruby.h>

#include <Ecore.h>
#include <Ecore_Evas.h>

#include "../ecore/rb_ecore.h"
#include "../ecore_x/rb_window.h"
#include "rb_ecore_evas_main.h"
#include "rb_ecore_evas.h"

static VALUE windows;

static VALUE c_new (int argc, VALUE *argv, VALUE klass)
{
	VALUE self, disp, parent, geom[4];
	Ecore_Evas **ee = NULL;
	char *cdisp = NULL;
	int i, igeom[4] = {0, 0, 0, 0};

	self = Data_Make_Struct (klass, Ecore_Evas *,
	                         NULL, c_ecore_evas_free, ee);

	rb_scan_args (argc, argv, "06", &disp, &parent,
	              &geom[0], &geom[1], &geom[2], &geom[3]);

	if (!NIL_P (disp)) {
		Check_Type (disp, T_STRING);
		cdisp = StringValuePtr (disp);
	}

	for (i = 0; i < 4; i++)
		if (!NIL_P (geom[i])) {
			Check_Type (geom[i], T_FIXNUM);
			igeom[i] = NUM2INT (geom[i]);
		}

	ecore_init ();
	ecore_evas_init ();

	*ee = ecore_evas_software_x11_new (cdisp, 0,
	                                   igeom[0], igeom[1],
	                                   igeom[2], igeom[3]);

	rb_obj_call_init (self, 0, NULL);

	return self;
}

static VALUE c_window_get (VALUE self)
{
	VALUE o;
	Ecore_X_Window w;

	GET_OBJ (self, Ecore_Evas *, ee);

	if (NIL_P (o = rb_hash_aref (windows, INT2NUM ((long) (ee))))) {
		w = ecore_evas_software_x11_window_get (*ee);
		o = TO_ECORE_X_WINDOW (self, w);
		rb_hash_aset (windows, INT2NUM ((long) ee), o);
	}

	return o;
}

void Init_SoftwareX11 (void)
{
	VALUE c;

	rb_require ("ecore_x");

	c = rb_define_class_under (mEvas, "SoftwareX11", cEcoreEvas);

	rb_define_singleton_method (c, "new", c_new, -1);
	rb_define_method (c, "window", c_window_get, 0);

	windows = rb_hash_new ();
	rb_global_variable (&windows);
}
